/*
   $Id: callbacks.c,v 1.9 2002/12/28 09:18:58 bhepple Exp $

   part of gjots - a gnome/X11 jotter

   Copyright (C) 2002 Robert Hepple

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <ctype.h>
#include <errno.h>
#include <sys/stat.h>
#include <gnome.h>
#include <time.h>
#include <regex.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "main.h"
#include "file.h"
#include "wrap.h"
#include "find.h"
#include "tree.h"

FILE		*cutBuffer = NULL;
GtkWidget 	*prefsDialog = NULL;

int 		readonlyCheckButtonGuard = 0;

void
unmsg() {
	gnome_appbar_pop(appbar);
}

void
msg(char *buf) {
	unmsg();
	gnome_appbar_push(appbar, buf);
}

/********* GENERAL DIALOG *********/

static int generalValue, generalNumFields;

static void
checkButton(GtkWidget *dialogue, int buttonMask, int mask, char *name) {
	GtkButton	*button;

	if (!(buttonMask & mask)) {
		button = (GtkButton *) lookup_widget(dialogue, name);
		if (button)
			gtk_widget_hide(GTK_WIDGET(button));
	}
}

/* Returns OK if OK or YES button is pressed */
int
doGeneralDialog(char *title,
				char *prompt,
				int	 buttonMask,
				int	 num_fields, /* 0, 1 or 2 */
				int  is_password,
				char *feedback,
				char *field1_label,
				char *field1_default,
				char *field1_retval,
				int  field1_retval_length,
				char *field2_label,
				char *field2_default,
				char *field2_retval,
				int  field2_retval_length) {

	GtkLabel	*generalPromptLabel, *generalLabel1, *generalLabel2, *generalFeedback;
	GtkWidget	*generalDialog;
	GtkEntry 	*generalField1, *generalField2;

	generalDialog = create_generalDialog();
	gtk_window_set_title(GTK_WINDOW(generalDialog), title);
	generalPromptLabel = (GtkLabel *)lookup_widget(generalDialog, "generalPromptLabel");
	if (generalPromptLabel)
		gtk_label_set_text(generalPromptLabel, prompt);

	generalField1 = (GtkEntry *)lookup_widget(generalDialog, "generalField1");
	generalLabel1 = (GtkLabel *)lookup_widget(generalDialog, "generalLabel1");
	generalField2 = (GtkEntry *)lookup_widget(generalDialog, "generalField2");
	generalLabel2 = (GtkLabel *)lookup_widget(generalDialog, "generalLabel2");
	if (!generalLabel1 || !generalField1 || !generalLabel2 || !generalField2)
		return CANCEL;

	generalNumFields = num_fields;

	switch (num_fields) {
	case 0:
		gtk_widget_hide(GTK_WIDGET(generalLabel1));
		gtk_widget_hide(GTK_WIDGET(generalField1));
	case 1:
		gtk_widget_hide(GTK_WIDGET(generalLabel2));
		gtk_widget_hide(GTK_WIDGET(generalField2));
	}

	switch (num_fields) {
	case 2:
		gtk_label_set_text(generalLabel2, field2_label);
		gtk_entry_set_text(generalField2, field2_default);
	case 1:
		gtk_label_set_text(generalLabel1, field1_label);
		gtk_entry_set_text(generalField1, field1_default);
	}

	if (is_password) {
		gtk_entry_set_visibility(generalField1, !is_password);
		gtk_entry_set_visibility(generalField2, !is_password);
	}

	generalFeedback = (GtkLabel *) lookup_widget(generalDialog, "generalFeedback");
	if (generalFeedback) {
		if (feedback && strcmp(feedback, ""))
			gtk_label_set_text(generalFeedback, feedback);
		else
			gtk_widget_hide(GTK_WIDGET(generalFeedback));
	}

	/* mask out buttons: */
	checkButton(generalDialog, buttonMask, YES, "generalYes");
	checkButton(generalDialog, buttonMask, NO, "generalNo");
	checkButton(generalDialog, buttonMask, OK, "generalOK");
	checkButton(generalDialog, buttonMask, CANCEL, "generalCancel");
	checkButton(generalDialog, buttonMask, READONLY, "generalReadonly");
	checkButton(generalDialog, buttonMask, TRYAGAIN, "generalTryagain");

	gtk_widget_show(generalDialog);
	generalValue = WAITING;
	while (generalValue == WAITING)
		gtk_main_iteration();

	if (generalValue == OK) {
		char *value;

		switch (num_fields) {
		case 2:
			value = gtk_editable_get_chars(GTK_EDITABLE(generalField2), 0, -1);
			strncpy(field2_retval, value, field2_retval_length);
			g_free(value);
		case 1:
			value = gtk_editable_get_chars(GTK_EDITABLE(generalField1), 0, -1);
			strncpy(field1_retval, value, field1_retval_length);
			g_free(value);
		}
	}

	gtk_widget_destroy(generalDialog);
	while (gtk_events_pending())
		gtk_main_iteration();
	return generalValue;
}

void
doWarning(char *prompt) {
	doGeneralDialog("gjots: WARNING!", prompt, OK, 0, 0, NULL,
					NULL, NULL, NULL, 0,
					NULL, NULL, NULL, 0);
}

int
doConfirm(char *prompt) {
	return(doGeneralDialog("gjots: confirm", prompt, YES|NO|CANCEL, 0, 0, NULL,
						   NULL, NULL, NULL, 0,
						   NULL, NULL, NULL, 0));
}

void
on_generalReadonly_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	generalValue = READONLY;
}

void
on_generalTryagain_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	generalValue = TRYAGAIN;
}

void
on_generalYes_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	generalValue = OK;

}

void
on_generalNo_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	generalValue = NO;

}

void
on_generalOK_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	generalValue = OK;
}

void
on_generalCancel_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	generalValue = CANCEL;
}


gboolean
on_generalField1_key_press_event       (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	GtkWidget *generalDialog;
	GtkEntry *generalField2;

	switch (event->keyval) {
	case GDK_Escape:
		generalValue = CANCEL;
		return TRUE;
	case GDK_KP_Enter:
	case GDK_Return:
		if (generalNumFields > 1) {
			generalDialog = gtk_widget_get_toplevel(widget);
			generalField2 = (GtkEntry *)lookup_widget(generalDialog, "generalField2");
			gtk_widget_grab_focus(GTK_WIDGET(generalField2));
		} else {
			generalValue = OK;
		}
		return TRUE;
	default:
		break;
	}
	return FALSE;
}


gboolean
on_generalField2_key_press_event       (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	switch (event->keyval) {
	case GDK_Escape:
		generalValue = CANCEL;
		return TRUE;
	case GDK_KP_Enter:
	case GDK_Return:
		generalValue = OK;
		return TRUE;
	default:
		break;
	}
	return FALSE;
}

/********* END GENERAL DIALOG *********/

static FILE *
newCutBuffer() {
	if (cutBuffer)
		fclose(cutBuffer); /* automatically deleted */
	disablePaste();
	return(tmpfile());
}

void
doPaste() {
	GList 		*selection = NULL;
	GtkWidget 	*item = NULL;

	if (readonly(READONLY_ENQ))
		return;

	syncPayload();
	/* Just take first item in selection: */
	selection = getSelectedItems();

	if (selection)
		item = (GtkWidget *)selection->data;
	if (!item || !GTK_IS_TREE_ITEM(item)) {
		msg("Nothing selected");
		goto cleanup;
	}

	if (cutBuffer) {
		fseek(cutBuffer, 0L, SEEK_SET);

		if (readItem(cutBuffer, GTK_TREE_ITEM(item)) == 0) {
			gnome_warning_dialog("Paste failed!");
			goto cleanup;
		}
		dirty_data = 1;
	}

 cleanup:
	if (selection)
		g_list_free(selection);
}

void
doCut() {
	GList 		*selection = NULL, *iterator;
	GtkWidget 	*item = NULL, *nextItem;
	GtkWidget 	*branch = NULL;

	if (readonly(READONLY_ENQ))
		return;

	syncPayload();
	selection = getSelectedItems();

	if (!selection || 
		!selection->data || 
		!GTK_WIDGET(selection->data)->parent) {
		msg("nothing selected!");
		goto cleanup;
	}

	if (GTK_TREE_ITEM(selection->data) == root) {
		msg("Can't delete root!");
		goto cleanup;
	}

	nextItem = getNextItem(GTK_WIDGET(selection->data), VISIBLE);
	branch = GTK_WIDGET(selection->data)->parent;

	cutBuffer = newCutBuffer();
	for (iterator = selection;
		 iterator;
		 iterator = iterator->next) {

		item = (GtkWidget *) iterator->data;
		if (GTK_IS_TREE_ITEM(item)) {
			if (writeItem(cutBuffer, GTK_TREE_ITEM(item))) {
				gnome_warning_dialog("Cut failed!");
				goto cleanup;
			}
			enablePaste();
			dirty_data = 1;
		}
	}

	gtk_tree_remove_items(GTK_TREE(branch), selection);
	if (nextItem) {
		gtk_tree_select_child(GTK_TREE(nextItem->parent), nextItem);
		gtk_widget_grab_focus(nextItem);
		gtk_widget_grab_default(nextItem);
	}
 cleanup:
	if (selection)
		g_list_free(selection);
}

void
doCopy() {
	GList 		*selection, *iterator;
	GtkWidget 	*item = NULL;

	syncPayload();
	cutBuffer = newCutBuffer();
	selection = getSelectedItems();
	for (iterator = selection;
		 iterator;
		 iterator = iterator->next) {

		item = (GtkWidget *) iterator->data;
		if (GTK_IS_TREE_ITEM(item)) {
			if (writeItem(cutBuffer, GTK_TREE_ITEM(item))) {
				gnome_warning_dialog("Paste failed!");
				goto cleanup;
			}
			enablePaste();
		}
	}
	msg("copied...");

 cleanup:
	if (selection)
		g_list_free(selection);
}

void
on_new_file1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	syncPayload();
	doNewFile();
}

void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	syncPayload();
	readFile(NULL);
	displayFileErrors();
}

void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	syncPayload();
	writeFile(0);
	displayFileErrors();
}

void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	syncPayload();
	writeFile(1);
	displayFileErrors();
}

void
on_readOnly_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!gjots) /* eg. at startup */
		return;

	if (!readonlyCheckButtonGuard)
		toggle_readonly();
}


void
quit_gjots()
{
	int		response;

	unmsg();
	syncPayload();
	if (dirty_data) {
		response = doConfirm("Save current file before exit?");
		if (response == CANCEL)
			return;
		if (response == OK) {
			syncPayload();
			writeFile(0);
			displayFileErrors();
		}
	}

	if (!readonly(READONLY_ENQ))
		unlock_file(NULL);
	write_rc_file();

	gtk_main_quit();
}

void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	quit_gjots();
}

void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	doCut();
}

void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	doCopy();
}

void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	doPaste();
}

void
on_clear1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	msg("This function is not yet implemented");

}

void
on_properties1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	msg("This function is not yet implemented");
}

void
on_prefsOK_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkSpinButton   *lineLength;
	GtkEntry		*externalEditor;
	GtkEntry		*dateFormat;
	GtkEntry		*fontName;

	lineLength = (GtkSpinButton *)lookup_widget(prefsDialog, "lineLength");
	if (lineLength) {
		prefs.lineLength = gtk_spin_button_get_value_as_int(lineLength);
	}

	externalEditor = (GtkEntry *)lookup_widget(prefsDialog, "externalEditor");
	if (externalEditor) {
		safe_strncpy(prefs.externalEditor, gtk_entry_get_text(externalEditor), MAXSTRING);
	}

	dateFormat = (GtkEntry *)lookup_widget(prefsDialog, "dateFormat");
	if (dateFormat) {
		safe_strncpy(prefs.dateFormat, gtk_entry_get_text(dateFormat), MAXSTRING);
	}

	fontName = (GtkEntry *)lookup_widget(prefsDialog, "fontName");
	if (fontName) {
		safe_strncpy(prefs.fontName, gtk_entry_get_text(fontName), MAXSTRING);
	}

	gtk_widget_destroy(prefsDialog);
	prefsDialog = NULL;
}


void
on_prefsCancel_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(prefsDialog);
	prefsDialog = NULL;
}

static
void doPrefs() {
	GtkSpinButton 	*lineLength;
	GtkEntry		*externalEditor;
	GtkEntry		*dateFormat;
	GtkEntry		*fontName;

	prefsDialog = create_prefs();

	lineLength = (GtkSpinButton *)lookup_widget(prefsDialog, "lineLength");
	if (lineLength) {
		gtk_spin_button_set_value(lineLength, (float) prefs.lineLength);
	}

	externalEditor = (GtkEntry *)lookup_widget(prefsDialog, "externalEditor");
	if (externalEditor) {
		gtk_entry_set_text(externalEditor,	prefs.externalEditor);
	}

	dateFormat = (GtkEntry *)lookup_widget(prefsDialog, "dateFormat");
	if (dateFormat) {
		gtk_entry_set_text(dateFormat, prefs.dateFormat);
	}

	fontName = (GtkEntry *)lookup_widget(prefsDialog, "fontName");
	if (fontName) {
		gtk_entry_set_text(fontName, prefs.fontName);
	}

	gtk_widget_show(prefsDialog);
}

void
on_prefs_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	doPrefs();
}

void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	gtk_widget_show (create_about());
}

void
on_read_manual_handler                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	char	buf[MAXBUF];

	sprintf(buf, "gjots -r " MANUALDIR "/readme.gjots &");
	puts(buf);
	system(buf);
}

/********* LEFT TOOLBAR *********/

void
on_newFolderButton_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	doNewFolder();
}

void
on_newPageButton_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	doNewItem();
}

void
on_upButton_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	moveItemsUp();
}

void
on_downButton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	moveItemsDown();
}

void
on_promoteButton_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	moveItemsLeft();
}

void
on_demoteButton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	moveItemsRight();
}

void
on_cutButton_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	doCut();
}

void
on_copyButton_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	doCopy();
}

void
on_pasteButton_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	doPaste();
}

/********* button-3 popup *********/

void
on_add_folder_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	doNewFolder();
}

void
on_add_item_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	doNewItem();
}

void
on_promote_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	moveItemsLeft();
}

void
on_demote_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	moveItemsRight();
}

void
on_up_activate                         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	moveItemsUp();
}

void
on_down_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	moveItemsDown();
}

void
on_cutMenuItem_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	doCut();
}

void
on_copyMenuItem_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	doCopy();
}

void
on_pasteMenuItem_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	doPaste();
}

/********* TOP MENUBAR *********/

void
on_openButton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	syncPayload();
	readFile(NULL);
	displayFileErrors();
}

void
on_saveButton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	syncPayload();
	writeFile(0);
	displayFileErrors();
}

void
on_optionsButton_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	unmsg();
	doPrefs();
}

#if FIND_WHERE_THE_ERRORS_ARE_COMING_FROM || 0
static inline gchar*
gtk_type_descriptive_name (GtkType type)
{
  gchar *name;

  name = gtk_type_name (type);
  if (!name)
    name = "(unknown)";

  return name;
}

GtkTypeObject*
gtk_type_check_object_cast (GtkTypeObject  *type_object,
			    GtkType         cast_type)
{
  if (!type_object)
    {
      g_warning ("invalid cast from (NULL) pointer to `%s'",
		 gtk_type_descriptive_name (cast_type));
      return type_object;
    }
  if (!type_object->klass)
    {
      g_warning ("invalid unclassed pointer in cast to `%s'",
		 gtk_type_descriptive_name (cast_type));
      return type_object;
    }
  /* currently, GTK_TYPE_OBJECT is the lowest fundamental type
   * dominator for types that introduce classes.
   */
  if (type_object->klass->type < GTK_TYPE_OBJECT)
    {
      g_warning ("invalid class type `%s' in cast to `%s'",
		 gtk_type_descriptive_name (type_object->klass->type),
		 gtk_type_descriptive_name (cast_type));
      return type_object;
    }
  if (!gtk_type_is_a (type_object->klass->type, cast_type))
    {
      g_warning ("invalid cast from `%s' to `%s'",
		 gtk_type_descriptive_name (type_object->klass->type),
		 gtk_type_descriptive_name (cast_type));
      return type_object;
    }

  return type_object;
}

#endif

void
on_import_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	syncPayload();
	doImportFile();
}

void
on_export_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	unmsg();
	syncPayload();
	doExportFile();
}

void
on_wrapButton_clicked            (GtkButton 	*button,
								  gpointer      user_data)
{
	compose_wrap_line(textDisplay);
}


gboolean
on_textDisplay_key_press_event         (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	if (event->state & GDK_CONTROL_MASK) {
		switch (event->keyval) {
		case GDK_L:
		case GDK_l:
			compose_wrap_line(textDisplay);
			return TRUE;
		}

	} else if (GDK_SHIFT_MASK & event->state) {

	} else if ((GDK_MODIFIER_MASK & event->state) == 0) { /* plain key */
		switch (event->keyval) {
		case GDK_F12:
			sendFocusToTree();
			return TRUE;
		}
	}
	return FALSE;
}

void
on_textDisplay_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkWidget *item;

	syncPayload();
	item = (GtkWidget *) gtk_object_get_user_data(GTK_OBJECT(textDisplay));
	checkTitle(item);
	clearFind();
}

void
on_textDisplay_move_cursor             (GtkEditable     *editable,
                                        gint             x,
                                        gint             y,
                                        gpointer         user_data)
{
	clearFind();
}


void
on_editorButton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar tmp_filename[64], *text;
	int tmp_file;
	struct stat statbuf_before, statbuf_after;
	static char *defcmd = "xterm -e vi %s";
	gchar buf[MAXSTRING];
	gchar *p;
	FILE *tmp;

	strcpy(tmp_filename, "/tmp/gjotsXXXXXX");
	tmp_file = mkstemp(tmp_filename);
	if (tmp_file < 0) {
		doWarning("Can't create temporary file on /tmp");
		return;
	}

	text = gtk_editable_get_chars(GTK_EDITABLE(textDisplay),
								  0,
								  gtk_text_get_length(textDisplay));

	if (text) {
		write(tmp_file, text, gtk_text_get_length(textDisplay));
	}
	close(tmp_file);

	if (prefs.externalEditor &&
	    (p = strchr(prefs.externalEditor, '%')) &&
	    *(p + 1) == 's' &&
		!strchr(p + 2, '%')) {
		sprintf(buf, prefs.externalEditor, tmp_filename);
	} else {
		if (prefs.externalEditor)
			doWarning("External editor command line is invalid, using default (xterm -e vi)");
		sprintf(buf, defcmd, tmp_filename);
	}

	errno = 0;
	if (stat(tmp_filename, &statbuf_before)) {
		doWarning("Error in stat on tmp_filename");
		goto ret;
	}

	if (system(buf)) {
		doWarning("Error running external editor");
	}

	errno = 0;
	if (stat(tmp_filename, &statbuf_after)) {
		doWarning("Error in stat on tmp_filename");
		goto ret;
	}

	if (statbuf_after.st_mtime == statbuf_before.st_mtime)
		goto ret;

	tmp = fopen(tmp_filename, "r");
	if (!tmp) {
		doWarning("Can't open tmpfile");
		goto ret;
	}

	clearTextDisplay();
	while (fgets(buf, MAXSTRING, tmp)) {
		gtk_text_insert(textDisplay, textDisplayFont, NULL, NULL, buf, -1);
	}
	syncPayload();
	fclose(tmp);

 ret:
	g_free(text);
	unlink(tmp_filename);
}

void
on_dateButton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	char 		buf[MAXBUF];
	struct tm 	*now_s;
	time_t		now;
	gint 		pos;

	time(&now);
	now_s = localtime(&now);
	strftime(buf, MAXBUF, prefs.dateFormat, now_s);
	gtk_text_freeze(textDisplay);
	pos = gtk_editable_get_position(GTK_EDITABLE(textDisplay));
	gtk_editable_insert_text(GTK_EDITABLE(textDisplay), buf, strlen(buf), &pos);
	gtk_text_thaw(textDisplay);
}


static int fontValue;

void
on_fontButton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget	*fontDialog;
	GtkEntry	*fontPrefsEntry;
	GdkFont		*newFont;

	fontDialog = create_fontselectiondialog();
	if (!fontDialog) return;

	fontPrefsEntry = (GtkEntry *)lookup_widget(prefsDialog, "fontName");
	if (!fontPrefsEntry) goto cleanup;

	gtk_font_selection_dialog_set_font_name((GtkFontSelectionDialog *)fontDialog,
											gtk_entry_get_text(fontPrefsEntry));

	gtk_widget_show(fontDialog);
	fontValue = WAITING;
	while (fontValue == WAITING)
		gtk_main_iteration();

	if (fontValue == OK) {
		gint	oldPosition;
		gchar	*oldText;
		gchar	*newName;

		newName = gtk_font_selection_dialog_get_font_name(
			(GtkFontSelectionDialog *)fontDialog);
		gtk_entry_set_text(fontPrefsEntry, newName);
		newFont = gdk_font_load(newName);
		if (!newFont) {
			msg("Can't load that font");
			goto cleanup;
		}
		safe_strncpy(prefs.fontName, newName, MAXSTRING);
		g_free(newName);

		textDisplayFont = newFont;
		gtk_text_freeze(textDisplay);
		oldPosition = gtk_editable_get_position(GTK_EDITABLE(textDisplay));
		oldText = gtk_editable_get_chars(GTK_EDITABLE(textDisplay), 0, -1);
		// gtk_editable_delete_text(GTK_EDITABLE(textDisplay), 0, -1);
		gtk_text_set_point(textDisplay, 0);
		gtk_text_forward_delete(textDisplay, gtk_text_get_length(textDisplay));
		gtk_text_insert(textDisplay, textDisplayFont, NULL, NULL, oldText, -1);
		gtk_editable_set_position(GTK_EDITABLE(textDisplay), oldPosition);
		gtk_text_thaw(textDisplay);
		g_free(oldText);
	}

 cleanup:
	gtk_widget_destroy(fontDialog);
	while (gtk_events_pending())
		gtk_main_iteration();
}


void
on_fontOK_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	fontValue = OK;
}


void
on_fontCancel_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	fontValue = CANCEL;
}


void
on_fontApply_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{

}


/********* FIND DIALOG CALLBACKS *********/

void
on_find1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	popupFind();
}


void
on_find_again1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	doFindNext(1);
}


void
on_findButton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	popupFind();
}


void
on_replaceStartButton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	doReplace();
}


void
on_replaceAllButton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	doReplaceAll();
	clearFind();
}


void
on_findDialog_destroy                  (GtkObject       *object,
                                        gpointer         user_data)
{
	zapFind();
}

void
on_findStartButton_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	doFindNext(1);
}


void
on_findCloseButton_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
	unPopFind(1);
}

void
on_findCancelButton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	unPopFind(0);
}

void
on_findClearButton_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	doFindClear();
}

gboolean
on_findEntry_key_press_event           (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	switch (event->keyval) {
	case GDK_Escape:
		unPopFind(0);
		return TRUE;
	case GDK_KP_Enter:
	case GDK_Return:
		doFindNext(1);
		return TRUE;
	default:
		break;
	}
	return FALSE;
}


gboolean
on_replaceEntry_key_press_event        (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	switch (event->keyval) {
	case GDK_Escape:
		unPopFind(0);
		return TRUE;
	case GDK_KP_Enter:
	case GDK_Return:
		doReplace();
		return TRUE;
	default:
		break;
	}
	return FALSE;
}

void
on_topToolbarCheck_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget	*w, *toolbar;

	if (!gjots) /* eg. at startup */
		return;

	w = lookup_widget(gjots, "topToolbarCheck");
	toolbar = lookup_widget(gjots, "toolbar1");
	if (w && toolbar) {
		if (GTK_CHECK_MENU_ITEM(w)->active)
			gtk_widget_show(toolbar);
		else
			gtk_widget_hide(toolbar);
	}
}

void
on_leftToolbarCheck_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget	*w, *toolbar;

	if (!gjots) /* eg. at startup */
		return;

	w = lookup_widget(gjots, "leftToolbarCheck");
	toolbar = lookup_widget(gjots, "alignment1");
	if (w && toolbar) {
		if (GTK_CHECK_MENU_ITEM(w)->active)
			gtk_widget_show(toolbar);
		else
			gtk_widget_hide(toolbar);
	}
}


void
on_gjots_destroy                       (GtkObject       *object,
                                        gpointer         user_data)
{
	quit_gjots();
}

